@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
-pub mod class_resolver;
 pub mod context;
 pub mod meta_resolver;
+pub mod meta_string_resolver;
+pub mod ref_resolver;
+pub mod type_resolver;
