@@ -19,9 +19,20 @@ use crate::ensure;
 use crate::error::Error;
 use crate::resolver::context::ReadContext;
 use crate::resolver::context::WriteContext;
+use crate::serializer::Serializer;
 use crate::types::TypeId;
 
-pub fn fory_write_data<T>(this: &[T], context: &mut WriteContext) -> Result<(), Error> {
+pub fn fory_write_data<T: Serializer>(this: &[T], context: &mut WriteContext) -> Result<(), Error> {
+    if context.is_xlang()
+        && matches!(
+            T::fory_static_type_id(),
+            TypeId::U8 | TypeId::U16 | TypeId::U32 | TypeId::U64 | TypeId::USIZE
+        )
+    {
+        return Err(Error::not_allowed(
+            "Unsigned types are not supported in cross-language mode",
+        ));
+    }
     let len_bytes = std::mem::size_of_val(this);
     context.writer.write_varuint32(len_bytes as u32);
 
